//////////////////////////////////////////////////////////////////////////////
//
// ***************************************************************************
// **                                                                       **
// ** Copyright (c) 1995-2004 Xilinx, Inc.  All rights reserved.            **
// **                                                                       **
// ** You may copy and modify these files for your own internal use solely  **
// ** with Xilinx programmable logic devices and Xilinx EDK system or       **
// ** create IP modules solely for Xilinx programmable logic devices and    **
// ** Xilinx EDK system. No rights are granted to distribute any files      **
// ** unless they are distributed in Xilinx programmable logic devices.     **
// **                                                                       **
// ***************************************************************************
//
//////////////////////////////////////////////////////////////////////////////
// Filename:          -\MyProcessorIPLib\drivers\opb_usb_ft245_v1_00_b\src\opb_usb_ft245.c
// Version:           1.00.b
// Description:       opb_usb_ft245 Driver Source File
// Date:              Wed Mar 16 10:08:06 2005 (by Create and Import Peripheral Wizard)
//////////////////////////////////////////////////////////////////////////////


/***************************** Include Files *******************************/

#include "opb_usb_ft245.h"

/************************** Function Definitions ***************************/

//********************************************************************************************
// This functions sends a single byte using the USB FT245 interface controller. It is blocking
// in that it waits for the transmitter fifo to become non-full before it writes the byte to
// FT245 controller.
//
// @param  - BaseAddress:	is the base address of the device
// @param  - Data:			is the byte of data to send
// @return - None.
// @note   - None.
//********************************************************************************************
void USB_FT245_SendByte(Xuint32 BaseAddress, Xuint8 Data)
{
	while(USB_FT245_mIsTransmitterFull(BaseAddress));
	USB_FT245_mWriteReg(BaseAddress, USB_FT245_TX_FIFO_OFFSET, Data);
}

//********************************************************************************************
// This functions receives a single byte using the UART. It is blocking in that it waits for
// the receiver fifo to become non-empty before it reads from the receive register.
//
// @param  - BaseAddress:	is the base address of the device
// @return - The byte of data received.
// @note   - None.
//********************************************************************************************
Xuint8 USB_FT245_RecvByte(Xuint32 BaseAddress)
{
	while(USB_FT245_mIsReceiverEmpty(BaseAddress));
	return(USB_FT245_mReadReg(BaseAddress, USB_FT245_RX_FIFO_OFFSET));
}

//********************************************************************************************
// This functions removes all data from receive FIFO
//
// @param  - BaseAddress:	is the base address of the device
// @return - None.
// @note   - None.
//********************************************************************************************
void USB_FT245_FlushRXFifo(Xuint32 BaseAddress)
{
	while(!USB_FT245_mIsReceiverEmpty(BaseAddress))
	{
		USB_FT245_mReadReg(BaseAddress, USB_FT245_RX_FIFO_OFFSET);
	}
}

