------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2004 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ** YOU MAY COPY AND MODIFY THESE FILES FOR YOUR OWN INTERNAL USE SOLELY  **
-- ** WITH XILINX PROGRAMMABLE LOGIC DEVICES AND XILINX EDK SYSTEM OR       **
-- ** CREATE IP MODULES SOLELY FOR XILINX PROGRAMMABLE LOGIC DEVICES AND    **
-- ** XILINX EDK SYSTEM. NO RIGHTS ARE GRANTED TO DISTRIBUTE ANY FILES      **
-- ** UNLESS THEY ARE DISTRIBUTED IN XILINX PROGRAMMABLE LOGIC DEVICES.     **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           1.00.b
-- Description:       User logic module.
-- Date:              Wed Mar 16 10:08:01 2005 (by Create and Import Peripheral Wizard)
-- VHDL-Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
-- 	active low signals:                    "*_n"
-- 	clock signals:                         "clk", "clk_div#", "clk_#x"
-- 	reset signals:                         "rst", "rst_n"
-- 	generics:                              "C_*"
-- 	user defined types:                    "*_TYPE"
-- 	state machine next state:              "*_ns"
-- 	state machine current state:           "*_cs"
-- 	combinatorial signals:                 "*_com"
-- 	pipelined or register delay signals:   "*_d#"
-- 	counter signals:                       "*cnt*"
-- 	clock enable signals:                  "*_ce"
-- 	internal version of output port:       "*_i"
-- 	device pins:                           "*_pin"
-- 	ports:                                 "- Names begin with Uppercase"
-- 	processes:                             "*_PROCESS"
-- 	component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

--library proc_common_v2_00_a;
--use proc_common_v2_00_a.proc_common_pkg.all;

-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Definition of Generics:
--   C_DWIDTH                     -- User logic data bus width
--   C_NUM_CE                     -- User logic chip enable bus width
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   Bus2IP_Data                  -- Bus to IP data bus for user logic
--   Bus2IP_BE                    -- Bus to IP byte enables for user logic
--   Bus2IP_RdCE                  -- Bus to IP read chip enable for user logic
--   Bus2IP_WrCE                  -- Bus to IP write chip enable for user logic
--   IP2Bus_Data                  -- IP to Bus data bus for user logic
--   IP2Bus_Ack                   -- IP to Bus acknowledgement
--   IP2Bus_Retry                 -- IP to Bus retry response
--   IP2Bus_Error                 -- IP to Bus error response
--   IP2Bus_ToutSup               -- IP to Bus timeout suppress
--
--------------------------------------------------------------------------------
-- Entity section
--------------------------------------------------------------------------------

entity user_logic is
	generic
	(
		-- ADD USER GENERICS BELOW THIS LINE ---------------
		--USER generics added here
		-- ADD USER GENERICS ABOVE THIS LINE ---------------

		-- DO NOT EDIT BELOW THIS LINE ---------------------
		-- Bus protocol parameters, do not add to or delete
		C_DWIDTH	: integer	:= 8;
		C_NUM_CE	: integer	:= 2
		-- DO NOT EDIT ABOVE THIS LINE ---------------------
	);
	port
	(
		-- ADD USER PORTS BELOW THIS LINE ------------------
		usb_data_I		: in std_logic_vector(0 to 7);			-- data bus input
		usb_data_O		: out std_logic_vector(0 to 7);			-- data bus	output
		usb_data_T		: out std_logic;								-- data bus tristate contol signal
		usb_RD_n			: out std_logic;								-- read (low active)
		usb_WR			: out std_logic;								-- write
		usb_TXE_n		: in std_logic;								-- transmiter fifo empty (low active)
		usb_RXF_n		: in std_logic;								-- receiver fifo full (low active)
		-- ADD USER PORTS ABOVE THIS LINE ------------------

		-- DO NOT EDIT BELOW THIS LINE ---------------------
		-- Bus protocol ports, do not add to or delete
		Bus2IP_Clk	: in	std_logic;
		Bus2IP_Reset	: in	std_logic;
		Bus2IP_Data	: in	std_logic_vector(0 to C_DWIDTH-1);
		Bus2IP_BE	: in	std_logic_vector(0 to C_DWIDTH/8-1);
		Bus2IP_RdCE	: in	std_logic_vector(0 to C_NUM_CE-1);
		Bus2IP_WrCE	: in	std_logic_vector(0 to C_NUM_CE-1);
		IP2Bus_Data	: out	std_logic_vector(0 to C_DWIDTH-1);
		IP2Bus_Ack	: out	std_logic;
		IP2Bus_Retry	: out	std_logic;
		IP2Bus_Error	: out	std_logic;
		IP2Bus_ToutSup	: out	std_logic
		-- DO NOT EDIT ABOVE THIS LINE ---------------------
	);
end entity user_logic;

--------------------------------------------------------------------------------
-- Architecture section
--------------------------------------------------------------------------------

architecture IMP of user_logic is
	type STATE_TYPE is (IDLE, READ, READ_DELAY, WRITE, DATA_HOLD, WRITE_DELAY);
 	signal CS: STATE_TYPE := IDLE;							-- current state
	signal data_reg_write_select : std_logic;
	signal data_reg_read_select : std_logic;
	signal data_reg_ack	: std_logic;
	signal status_reg_write_select : std_logic;
	signal status_reg_read_select : std_logic;
	signal delay : std_logic_vector (2 downto 0);

begin
	usb_data_O <= Bus2IP_Data when data_reg_write_select = '1' else (others=>'0');

	usb_data_T <= not(data_reg_write_select);
	IP2Bus_Data <= usb_data_I when data_reg_read_select = '1' else
					   ("000000" & usb_RXF_n & usb_TXE_n) when status_reg_read_select = '1' else (others=>'0');

	IP2Bus_Error       <= '0';
	IP2Bus_Retry       <= '0';
	IP2Bus_Ack         <= data_reg_ack or status_reg_write_select or status_reg_read_select;

	data_reg_write_select <= Bus2IP_WrCE(1);
	data_reg_read_select  <= Bus2IP_RdCE(1);
	status_reg_write_select <= Bus2IP_WrCE(0);
	status_reg_read_select  <= Bus2IP_RdCE(0);

	SLAVE_ACCESS_PROC : process( Bus2IP_Clk ) is
	begin
		if Bus2IP_Clk'event and Bus2IP_Clk = '1' then		-- rising edge
			if Bus2IP_Reset = '1' then								-- reset
				CS <= IDLE;
				data_reg_ack <= '0';
				IP2Bus_ToutSup <= '0';
				delay <= (others=>'0');
			else
				CS <= CS;
				case CS is
					when IDLE =>
						usb_RD_n <= '1';
						usb_WR <= '0';
						data_reg_ack <= '0';
						IP2Bus_ToutSup <= '0'; 

				      if data_reg_write_select = '1' then
							IP2Bus_ToutSup <= '1';
							if usb_TXE_n ='0' then
								CS <= WRITE;
								delay <= (others => '1');
								usb_RD_n <= '1';
								usb_WR <= '1';
							else
								CS <= IDLE;
							end if;
						elsif data_reg_read_select = '1' then
							IP2Bus_ToutSup <= '1';
							if usb_RXF_n ='0' then
								CS <= READ_DELAY;
								delay <= (others => '1');
								usb_RD_n <= '0';
								usb_WR <= '0';
							else
								CS <= IDLE;
							end if;
						end if;

					when READ_DELAY =>
						if delay = "000" then
							data_reg_ack <= '1';
							CS <= READ;
						else
							delay <= delay - "001";
						end if;

					when READ =>
						usb_RD_n <= '1';
						usb_WR <= '0';
						data_reg_ack <= '0';
						IP2Bus_ToutSup <= '0'; 
						CS <= IDLE;

					when WRITE =>
						if delay = "000" then
							--delay loop
							usb_RD_n <= '1';
							usb_WR <= '0';
							CS <= DATA_HOLD;
						else
							delay <= delay - "001";
						end if;

					when DATA_HOLD =>
						data_reg_ack <= '1';
						CS <= WRITE_DELAY;

					when WRITE_DELAY =>
						usb_RD_n <= '1';
						usb_WR <= '0';
						data_reg_ack <= '0';
						IP2Bus_ToutSup <= '0';
						CS <= IDLE;
   	
			 	end case;
			end if;
 		end if;
	end process SLAVE_ACCESS_PROC;
end IMP;
